library(plotrix)
library(mvtnorm)

## Input values for discretized optimization problem

max_error_prob <- 0
alpha <- (0.05 - 10^(-4)) ## desired FWER
beta <- (1-0.88) ## 1 minus beta is desired power to reject H0C at ncp_alt defined below
p1<- 0.5 # proportion of population in subpopulation 1
p2<- 1-p1 # proportion of population in subpopulation 2
rho1 <- sqrt(p1) # correlation of Z_1 and Z_C
rho2 <- sqrt(p2) # correlation of Z_2 and Z_C
ncp_alt <- (qnorm(1-alpha) + qnorm(0.9))*c(rho1,rho2) # alternative used in power constraint which is set so that UMP test of H0C alone has power 0.9 to reject, at level alpha. 
delta_1_min <- ncp_alt[1]-(10^(-10)) # set just below alternative used in power constraint, so as to be compatible with loss function and prior below
delta_2_min <- ncp_alt[2]-(10^(-10))
#set widths of large square outside of which multiple testing procedure rejects no null hypotheses (corresponds to w in text)
w1 <- 5  
w2 <- 5
# dimension of small squares used in discretization
tau <- 0.02
number_actions <- 6 # number of actions excluding default action 0
					# These correspond to rejecting the subsets of null hypotheses: 
					# S = (H01,H02,H0C,{H01,H0C},{H02,H0C},{H01,H02,H0C}) 

map_from_P_to_type_I_error_indicator_over_set_of_actions <- function(ncp)
{
	return(c(ncp[1]<=0,ncp[2]<=0,rho1*ncp[1]+rho2*ncp[2]<=0,ncp[1]<=0 || rho1*ncp[1]+rho2*ncp[2]<=0,ncp[2]<=0 || rho1*ncp[1]+rho2*ncp[2]<=0,ncp[1]<=0 || ncp[2]<=0 || rho1*ncp[1]+rho2*ncp[2]<=0))
}

indicator_contribute_to_combined_pop_power <- c(0,0,1,1,1,1)

# Loss function
L <- function(ncp){ # returns vector corresponding to loss function values at each action, for data generating distribution defined by non-centrality parameters ncp
	L1 <- as.numeric(ncp[1]>delta_1_min)*c(1,0,1,1,0,1,0) # indicator of delta_1 > delta_1_min and subset S_j (defined above) containing H_{01}
	L2 <- as.numeric(ncp[2]>delta_2_min)*c(1,1,0,1,1,0,0) # indicator of delta_2 > delta_2_min and subset S_j containing H_{02}
	L_total <- L1+L2
	return(L_total[2:7] - L_total[1]) # subtract L_total[1] since first element represents default option, which we do not allocate a variable for in our LP representation.
}

# For Symmetric Case:
prior_weights <- c(1/4,1/4,1/4,1/4)
prior_support <- c(list(c(0,0)),list(c(ncp_alt[1],0)),list(c(0,ncp_alt[2])),list(ncp_alt))

# list of pairs of non-centrality parameters in G_{tau,w}
ncp_list <- list()
for(x1 in seq(-w1,w2,by=tau)) {ncp_list <- c(ncp_list,list(c(x1,0)))}
for(y in seq(-w1,w2,by=tau)) {ncp_list <- c(ncp_list,list(c(0,y)))}
for(z in seq(-w1/rho2,w2/rho1,by=tau)) {ncp_list <- c(ncp_list,list(c(rho2*z,-rho1*z)))}
ncp_list <- unique(ncp_list)

# Settings for integration region and precision in computing lower bound to original problem using dual solution to discretized problem
max_eval_iters <- 100000
w1_unconst <- 10
w2_unconst <- 10

# Set grid density for grid search used in computing worst-case familywise Type I error rate
# Each rectangle in grid set to have dimensions:
grid_width_1 <- 10^(-4)
grid_width_2 <- 10^(-4)

load("list_of_rectangles.rdata")

lower_left_coordinates_for_list_of_rectangles <- array(0,c(length(list_of_rectangles),2))
for(i in 1:length(list_of_rectangles)){lower_left_coordinates_for_list_of_rectangles[i,] <- list_of_rectangles[[i]]}

## Solution x to discretized LP based on sparse LP solver is assumed stored in file solution_discretized_LP
x <- read.table("solution_discretized_LP")

## determine number of rectangles where probability mass spread out among multiple subsets of null hypotheses:
counter <- 1
spread_list <- c()
for(i in 1:(dim(x)[1]/number_actions))
{
	probs <- x[(number_actions*(i-1)+1):(number_actions*i),1]
	if(sum(probs) - max(probs)>0.02){ counter<- counter +1; spread_list <- c(spread_list,i); 
	print(c(lower_left_coordinates_for_list_of_rectangles[i,],probs))}
}
print(counter)

# round so that all values 0 or 1 and store as vector x_rounded
x_rounded <- array(0,dim(x))
for(i in 1:(dim(x)[1]/number_actions))
{
	probs <- x[(number_actions*(i-1)+1):(number_actions*i),1]
	if(max(probs)>0.75){
		# determine which index correponds to this max value:
		j <- which(probs==max(probs))	
		x_rounded[(number_actions*(i-1)+j),1] <- 1
		} # otherwise default is leave all at 0
}

### Make plot of solution for importing to paper:

postscript(paste("plot_optimal_rejection_region.eps"),height=8,horizontal=FALSE,onefile=FALSE,width=8)
par(mar=c(5.1, 5.1, 4.1, 1.1))
plot(0,xlim=c(-4,4),ylim=c(-4,4),type="n",xlab="Z-statistic for Population 1",ylab="Z-statistic for Population 2",main=expression(paste("b. Rejection Regions for ",m[sym]^"*","(0.88)",sep="")),cex.lab=1.8, cex.axis=1.8, cex.main=1.8, cex.sub=1.8)

for(j in 1:number_actions)
{
rectangles_where_hyp_subset_j_rejected <- c()
for(i in 1:(dim(x)[1]/number_actions))
{
	probs <- x_rounded[(number_actions*(i-1)+1):(number_actions*i),1]
	if(probs[j]==1){rectangles_where_hyp_subset_j_rejected <- c(rectangles_where_hyp_subset_j_rejected,i)}
}	
points(lower_left_coordinates_for_list_of_rectangles[rectangles_where_hyp_subset_j_rejected,],xlim=c(-5,5),ylim=c(-5,5),col=j,pch=20,cex=0.25)
}
#abline(a=qnorm(0.95)*sqrt(2),b=(-1),xlim=c(-3,5))
plot_max_val <- 5
intercept_val <- qnorm(0.95)*sqrt(2)
slope_val <-(-1)
x0 <- (plot_max_val-intercept_val)/slope_val
y0 <- plot_max_val
x1 <- plot_max_val
y1 <- intercept_val + slope_val*plot_max_val
segments(x0,y0,x1,y1)


#points(lower_left_coordinates_for_list_of_rectangles[spread_list[],],xlim=c(-5,5),ylim=c(-5,5),col=7,pch=20)

df1 <- data.frame("Reject\n H02")
addtable2plot(x=-4.2,y=3,table=df1,bg="white",bty="o",hlines=FALSE,display.colnames=FALSE,cex=1.5)
df1 <- data.frame("Reject\n H02,H0C")
addtable2plot(x=-0.3,y=3,table=df1,bg="white",bty="o",hlines=FALSE,display.colnames=FALSE,cex=1.5)
df1 <- data.frame("Reject\n H01,H02,\n H0C")
addtable2plot(x=2.5,y=2.4,table=df1,bg="white",bty="o",hlines=FALSE,display.colnames=FALSE,cex=1.5)
df1 <- data.frame("Reject\n H01,H0C")
addtable2plot(x=2.5,y=0,table=df1,bg="white",bty="o",hlines=FALSE,display.colnames=FALSE,cex=1.5)
df1 <- data.frame("Reject\n H01")
addtable2plot(x=2.5,y=-3,table=df1,bg="white",bty="o",hlines=FALSE,display.colnames=FALSE,cex=1.5)
df1 <- data.frame("Reject\n H0C")
addtable2plot(x=0,y=0,table=df1,bg="white",bty="o",hlines=FALSE,display.colnames=FALSE,cex=1.5)
arrows(0.5,1,1.35,1.35,angle=15)
df1 <- data.frame("Reject\n Nothing")
addtable2plot(x=-2,y=-2,table=df1,bg="white",bty="n",hlines=FALSE,display.colnames=FALSE,cex=1.5)
arrows(0.5,1,1.35,1.35,angle=15)
dev.off()


## DUAL PLOT, Single plot output for inclusion in paper

postscript(paste("plot_dual.eps"),height=8,horizontal=FALSE,onefile=FALSE,width=8)
par(mar=c(5.1, 5.1, 4.1, 1.1))
plot(0,xlim=c(-4,4),ylim=c(-4,4),type="n",xlab=expression(paste("Non-centrality Parameter ",delta[1]," for Population 1")),ylab=expression(paste("Non-centrality Parameter  ",delta[2],"  for Population 2")),main=expression(paste("b. Active Constraints for ",m[sym]^"*","(0.88)",,sep="")),cex.lab=1.8, cex.axis=1.8, cex.main=1.8, cex.sub=1.8)
dual_dense <- read.table("dual_dense")
dual_list <- ncp_list[which(dual_dense>0.000001)]
print(dual_list)
for(i in 1:length(dual_list)){points(dual_list[[i]][1],dual_list[[i]][2],pch=16,cex=2)}
points(ncp_alt[1],ncp_alt[2],pch=2,cex=2)
legend("bottomleft",legend=c("Type I Error","Power"),title = "Constraint Type:",pch=c(16,2),cex=1.8)

## G_tau lines
plot_max_val <- 5
intercept_val <- 0
slope_val <-(-rho1/rho2)
x0 <- (plot_max_val-intercept_val)/slope_val
y0 <- plot_max_val
x1 <- plot_max_val
y1 <- intercept_val + slope_val*plot_max_val
segments(x0,y0,x1,y1)
segments(-plot_max_val,0,plot_max_val,0)
segments(0,-plot_max_val,0,plot_max_val)

dev.off()

postscript("plot_summary_optimal_solution.eps",height=8,horizontal=FALSE,onefile=FALSE,width=8)
par(mar=c(5.1, 5.1, 4.1, 4.1))
par(mfrow=c(2,2))
plot(0,xlim=c(-w1,w1),ylim=c(-w2,w2),type="n",xlab="Z-statistic for Population 1",ylab="Z-statistic for Population 2",main=paste("Fig.1a. Optimal Rejection Regions\n at Equal Subpopulation Fractions"))
## loop over all possible actions and plot where it is optimal to take each action
for(j in 1:number_actions)
{
rectangles_where_hyp_subset_j_rejected <- c()
for(i in 1:(dim(x)[1]/number_actions))
{
	probs <- x_rounded[(number_actions*(i-1)+1):(number_actions*i),1]
	if(probs[j]==1){rectangles_where_hyp_subset_j_rejected <- c(rectangles_where_hyp_subset_j_rejected,i)}
}	
points(lower_left_coordinates_for_list_of_rectangles[rectangles_where_hyp_subset_j_rejected,],xlim=c(-5,5),ylim=c(-5,5),col=j,pch=20)
}
#abline(a=qnorm(0.95)*sqrt(2),b=(-1),xlim=c(-3,5))
plot_max_val <- 5
intercept_val <- qnorm(0.95)*sqrt(2)
slope_val <-(-1)
x0 <- (plot_max_val-intercept_val)/slope_val
y0 <- plot_max_val
x1 <- plot_max_val
y1 <- intercept_val + slope_val*plot_max_val
segments(x0,y0,x1,y1)

points(lower_left_coordinates_for_list_of_rectangles[spread_list,],xlim=c(-5,5),ylim=c(-5,5),col=7,pch=20,cex=0.5)

df1 <- data.frame("Reject\n H02")
addtable2plot(x=-3,y=3,table=df1,bg="white",bty="o",hlines=FALSE,display.colnames=FALSE,cex=0.5)
df1 <- data.frame("Reject\n H02,H0C")
addtable2plot(x=-0.3,y=3,table=df1,bg="white",bty="o",hlines=FALSE,display.colnames=FALSE,cex=0.5)
df1 <- data.frame("Reject\n H01,H02,\n H0C")
addtable2plot(x=2.5,y=2.4,table=df1,bg="white",bty="o",hlines=FALSE,display.colnames=FALSE,cex=0.5)
df1 <- data.frame("Reject\n H01,H0C")
addtable2plot(x=2.5,y=0,table=df1,bg="white",bty="o",hlines=FALSE,display.colnames=FALSE,cex=0.5)
df1 <- data.frame("Reject\n H01")
addtable2plot(x=2.5,y=-3,table=df1,bg="white",bty="o",hlines=FALSE,display.colnames=FALSE,cex=0.5)

## DUAL PLOT
plot(0,xlim=c(-w1,w1),ylim=c(-w2,w2),type="n",xlab=expression(paste("Non-centrality Parameter ",delta[1]," for Population 1")),ylab=expression(paste("Non-centrality Parameter  ",delta[2],"  for Population 2")),main="Fig.1b. Active Type I Error Constraints\n and Power Constraint")
dual_dense <- read.table("dual_dense")
dual_list <- ncp_list[which(dual_dense>0.000001)]
print(dual_list)
for(i in 1:length(dual_list)){points(dual_list[[i]][1],dual_list[[i]][2],pch=16)}
points(ncp_alt[1],ncp_alt[2],pch=2)
legend("bottomleft",legend=c("Type I Error","Power"),title = "Constraint Type:",pch=c(16,2))

## G_tau lines
plot_max_val <- 5
intercept_val <- 0
slope_val <-(-rho1/rho2)
x0 <- (plot_max_val-intercept_val)/slope_val
y0 <- plot_max_val
x1 <- plot_max_val
y1 <- intercept_val + slope_val*plot_max_val
segments(x0,y0,x1,y1)
segments(-plot_max_val,0,plot_max_val,0)
segments(0,-plot_max_val,0,plot_max_val)

## Vector to keep track of key performance metrics
performance_vector <- rep(0,5)

## Get risk at our m*:
c_vector <- scan("c.txt")
print(t(x) %*% c_vector)
performance_vector[3] <- t(x) %*% c_vector

## Get components contributing to risk for m*:

components_contributing_to_risk <- rep(0,length(prior_support))
counter <- 1
for(ncp in prior_support)
{
  contribution_to_risk <- c()
  loss_function_value <- L(ncp) # returns vector of loss function value for each possible action
  for(r in list_of_rectangles){
    ## Get P_{ncp_vec}(Z \in r)
    prob_r <- pmvnorm(mean=ncp,sigma=diag(2),lower=c(r[1],r[2]),upper=c(r[1]+tau,r[2]+tau))
    ## Get subsets at which H_{01} rejected	
    if(attr(prob_r,"error") > max_error_prob){max_error_prob <- attr(prob_r,"error")}
    contribution_to_risk <- c(contribution_to_risk,prob_r)
  }
  components_contributing_to_risk[counter] <- t(as.vector(loss_function_value %*% t(contribution_to_risk))) %*% as.matrix(x)
  counter <- counter + 1
}
print("components of risk of m*")
print(components_contributing_to_risk)
print("with weights")
print(prior_weights)
## double check:
print(t(components_contributing_to_risk) %*% prior_weights)
## this should equal risk at m*:
print(t(x) %*% c_vector)

## get power of discretized procedure at H_0C alternative:

power_at_ncp_alt_rej_H0C <- 0
i <- 1
for(r in list_of_rectangles){

	M_probs <- x[(number_actions*(i-1)+1):(number_actions*i),1]
	## Get P_{ncp_vec}(Z \in r)
	prob_r <- pmvnorm(mean=ncp_alt,sigma=diag(2),lower=c(r[1],r[2]),upper=c(r[1]+tau,r[2]+tau))
	power_at_ncp_alt_rej_H0C <- power_at_ncp_alt_rej_H0C + prob_r * (t(M_probs)%*%indicator_contribute_to_combined_pop_power)
	i <- i + 1
}
performance_vector[1] <- power_at_ncp_alt_rej_H0C

risk_values_symmetric_m_088 <- c(power_at_ncp_alt_rej_H0C,components_contributing_to_risk[2:4],-t(x) %*% c_vector)
save(risk_values_symmetric_m_088,file="risk_values_symmetric_m_088.rdata")


## Construct n x 2 matrix of (Delta1,Delta2) pairs corresponding to tight Type I error constraints
dual_variable_values <- read.table("dual_dense")[,1]
if(length(ncp_list)!=(length(dual_variable_values)-1)){print("error"); stop();}
dual_variables_for_Type_I_error_constraints <- dual_variable_values[1:(length(dual_variable_values)-1)]
dual_variable_for_power_constraint <- dual_variable_values[length(dual_variable_values)]
active_Type_I_error_constraints <- ncp_list[which(dual_variables_for_Type_I_error_constraints>0.000001)]
dual_variables_for_active_Type_I_error_constraints <-dual_variables_for_Type_I_error_constraints[which(dual_variables_for_Type_I_error_constraints>0.000001)]
power_constraint <- ncp_alt

print(active_Type_I_error_constraints)
print(dual_variables_for_active_Type_I_error_constraints)
print(dual_variable_for_power_constraint)
print(power_constraint)

# get worst case familywise Type I error over all constraints in G_{tau,w}, and put * next to those corresponding to active constraints

max_fwer <- 0
ncp_counter <- 1
active_constraint_counter <- 1
FWER_ACHIEVED <- rep(0,length(active_Type_I_error_constraints)) # Keep track of Familywise Type I error rate at each such active constraint, since it may be slightly less than alpha; this is used in computing the lower bound via the dual below
for(ncp in ncp_list){
  FWER <- 0
  i <- 1
  indicator_contribute_to_FWER <- map_from_P_to_type_I_error_indicator_over_set_of_actions(ncp)
  for(r in list_of_rectangles){
    M_probs <- x[(number_actions*(i-1)+1):(number_actions*i),1]
    ## Get P_{ncp_vec}(Z \in r)
    prob_r <- pmvnorm(mean=ncp,sigma=diag(2),lower=c(r[1],r[2]),upper=c(r[1]+tau,r[2]+tau))
    FWER <- FWER + prob_r * (t(M_probs)%*%indicator_contribute_to_FWER)
    i <- i + 1
  }
  if(FWER[1] > max_fwer){max_fwer <- FWER[1]}
  if(dual_variables_for_Type_I_error_constraints[ncp_counter]>0.000001) {print(c("Active FWER constraint:",FWER[1])); FWER_ACHIEVED[active_constraint_counter] <- FWER[1]; active_constraint_counter <- active_constraint_counter+1} else print(FWER[1])
  ncp_counter <- ncp_counter +1
}
performance_vector[2] <- max_fwer


#### 
#### Determine approximation error using dual solution to discretized LP
#### 

## Function M_u mapping (z1,z2) to optimal choice for unconstrainted problem.

M_u <- function(z1,z2){
	minimizing_s_index <- 0
	minimizing_value_cond_on_z_1_and_z_2 <- 0 # this is set to represent value of (21) at default action of rejecting no null hypothesis; consider term in (21):
	# first term: no contribution, since we standardize loss function so equals zero at default action
	# second term: no contribution, since default action doesn't reject H0C
	# third term: no contribution, since cannot make familywise Type I error under default action or rejecting no null hypotheses

	for(s_index in 1:number_actions) #compute value of (21) at each possible action, with goal of finding optimal action to minimize risk of unconstrained problem conditioned on (z_1,z_2):
	{
		value_cond_on_z_1_and_z_2 <- 0
		counter <- 1
		for(ncp in prior_support) ## first term: integral with respect to prior
		{
			value_cond_on_z_1_and_z_2 <- value_cond_on_z_1_and_z_2 + L(ncp)[s_index]*dnorm(z1-ncp[1])*dnorm(z2-ncp[2])*prior_weights[counter]
			counter <- counter + 1
		}
		# second term: contribution from power constraint
		if(indicator_contribute_to_combined_pop_power[s_index]==TRUE){
			value_cond_on_z_1_and_z_2 <- value_cond_on_z_1_and_z_2 - dual_variable_for_power_constraint*dnorm(z1-power_constraint[1])*dnorm(z2-power_constraint[2])
		}
		# third term: contribution from active familywise Type I error constraints for which a familywise Type I error occurs under action s
		for(i in 1:length(dual_variables_for_active_Type_I_error_constraints))
		{
			ncp <- active_Type_I_error_constraints[[i]]
			if(map_from_P_to_type_I_error_indicator_over_set_of_actions(ncp)[s_index]==TRUE){
				value_cond_on_z_1_and_z_2 <- value_cond_on_z_1_and_z_2 + dual_variables_for_active_Type_I_error_constraints[i]*dnorm(z1-ncp[1])*dnorm(z2-ncp[2])
			}
		}
		#print(value_cond_on_z_1_and_z_2)
		if(value_cond_on_z_1_and_z_2 < minimizing_value_cond_on_z_1_and_z_2){minimizing_s_index <- s_index;	minimizing_value_cond_on_z_1_and_z_2 <- value_cond_on_z_1_and_z_2}
	}
	return(c(minimizing_s_index,minimizing_value_cond_on_z_1_and_z_2))
}


plot(0,xlim=c(-w1,w1),ylim=c(-w2,w2),type="n",xlab=expression(paste("Non-centrality Parameter ",delta[1]," for Population 1")),ylab=expression(paste("Non-centrality Parameter  ",delta[2],"  for Population 2")),main="Plot of M_u")

plot_points <- array(0,c(10201,2))
color_vector <- rep(0,10201)
counter <- 1
for(z1 in seq(-w1,w1,length=101))
{
	for(z2 in seq(-w2,w2,length=101))
	{

		plot_points[counter,] <- c(z1,z2)
		color_vector[counter] <- M_u(z1,z2)[1]		
		counter <- counter +1		
	}
}
points(plot_points,col=color_vector,pch=16)


## Approximate lower bound (20) evaluated at M_u^*, method 1:
library(cubature)
function_to_integrate <- function(z){
  return(M_u(z1=z[1],z2=z[2])[2]) # value at M_u^* at z_1,z_2
}
integral_value <- adaptIntegrate(function_to_integrate,c(-w1_unconst,-w2_unconst),c(w1_unconst,w2_unconst),maxEval=max_eval_iters)
sum_value <- 0 # Compute contribution from each nu_j*(-alpha)
for(i in 1:length(dual_variables_for_active_Type_I_error_constraints))
{
  sum_value <- sum_value - dual_variables_for_active_Type_I_error_constraints[i]*FWER_ACHIEVED[i]
}
print("contributions to, and total, risk for M_u^*")
print(c(integral_value$integral,dual_variable_for_power_constraint*(power_at_ncp_alt_rej_H0C),sum_value,integral_value$integral + dual_variable_for_power_constraint*(power_at_ncp_alt_rej_H0C) + sum_value))

## Approximate lower bound (20) evaluated at M_u^*, method 2:
max_integration_error_prob <- 0
## first term:
value_of_unconstrained_problem_at_M_u <- 0
## first term: integral with respect to prior
function_to_integrate <- function(z,ncp){
	M_u_optimal_action <- M_u(z1=z[1],z2=z[2])[1]
	return(dnorm(ncp[1]-z[1])*dnorm(ncp[2]-z[2])*L(ncp)[M_u_optimal_action])	
}
counter <- 1
for(ncp in prior_support) 
{
	integral_value <- adaptIntegrate(function_to_integrate,c(-w1_unconst,-w2_unconst),c(w1_unconst,w2_unconst),ncp,maxEval=max_eval_iters)
	if(max(integral_value$error) > max_integration_error_prob){max_integration_error_prob <- max(integral_value$error)}
	value_of_unconstrained_problem_at_M_u <- value_of_unconstrained_problem_at_M_u + prior_weights[counter]*integral_value$integral
	counter <- counter +1
}
print(value_of_unconstrained_problem_at_M_u)
## second term
function_to_integrate <- function(z,ncp){
	M_u_optimal_action <- M_u(z1=z[1],z2=z[2])[1]
	return(ifelse(M_u_optimal_action==0,0,dnorm(ncp[1]-z[1])*dnorm(ncp[2]-z[2])*indicator_contribute_to_combined_pop_power[M_u_optimal_action]))
}
integral_value <- adaptIntegrate(function_to_integrate,c(-w1_unconst,-w2_unconst),c(w1_unconst,w2_unconst),ncp=ncp_alt,maxEval=max_eval_iters)
power_H0C <- integral_value$integral
print("power H0C under M_u")
print(power_H0C)
if(max(integral_value$error) > max_integration_error_prob){max_integration_error_prob <- max(integral_value$error)}
value_of_unconstrained_problem_at_M_u <- value_of_unconstrained_problem_at_M_u + dual_variable_for_power_constraint*(power_at_ncp_alt_rej_H0C - power_H0C)
print(value_of_unconstrained_problem_at_M_u)
## third term
print("familywise Type I error under M_u at active Type I error constraints")
function_to_integrate <- function(z,ncp){
	M_u_optimal_action <- M_u(z1=z[1],z2=z[2])[1]
	return(ifelse(M_u_optimal_action==0,0,dnorm(ncp[1]-z[1])*dnorm(ncp[2]-z[2])*map_from_P_to_type_I_error_indicator_over_set_of_actions(ncp)[M_u_optimal_action]))
}
for(i in 1:length(dual_variables_for_active_Type_I_error_constraints))
{
	ncp <- active_Type_I_error_constraints[[i]]
	integral_value <- adaptIntegrate(function_to_integrate,c(-w1_unconst,-w2_unconst),c(w1_unconst,w2_unconst),ncp,maxEval=max_eval_iters)
	Familywise_Type_I_error <- integral_value$integral
	print(c(ncp,Familywise_Type_I_error))
	if(max(integral_value$error) > max_integration_error_prob){max_integration_error_prob <- max(integral_value$error)}
value_of_unconstrained_problem_at_M_u <- value_of_unconstrained_problem_at_M_u + dual_variables_for_active_Type_I_error_constraints[i]*(Familywise_Type_I_error-FWER_ACHIEVED[i])
}
print(value_of_unconstrained_problem_at_M_u)
print(max_integration_error_prob)

performance_vector[4]<-value_of_unconstrained_problem_at_M_u
performance_vector[5]<-max_integration_error_prob

write(performance_vector,f=paste("summary_performance_of_solution"))

summary_output <- floor(performance_vector*1000)/1000
plot(0,type='n',main=paste("Power H0C, Max Type I error, Risk, Risk Lower Bound, Integr. Error\n",summary_output[1],summary_output[2],summary_output[3],summary_output[4],summary_output[5]),xlab="Plot intentionally blank")

dev.off()

# evaluate Type I error at active FWER constraints:


# do more thorough evaluation of worst case familywise Type I error over finer grid of non-centrality parameters in set G
max_fwer <- 0
ncp_max_fwer <- c(0,0)
print("doing dense grid search to compute worst case familywise Type I error rate")
get_familywise_Type_I_error_rate <- function(ncp)
  {
    FWER <- 0
    i <- 1
    indicator_contribute_to_FWER <- map_from_P_to_type_I_error_indicator_over_set_of_actions(ncp)
    for(r in list_of_rectangles){
      M_probs <- x[(number_actions*(i-1)+1):(number_actions*i),1]
      ## Get P_{ncp_vec}(Z \in r)
      prob_r <- pmvnorm(mean=ncp,sigma=diag(2),lower=c(r[1],r[2]),upper=c(r[1]+tau,r[2]+tau))
      FWER <- FWER + prob_r * (t(M_probs)%*%indicator_contribute_to_FWER)
      i <- i + 1
    }
    return(FWER)
  }

for(delta1 in seq(-w1,w1,by=grid_width_1))
  {
    ncp <- c(delta1,0)
    fwer <- get_familywise_Type_I_error_rate(ncp)
    if(fwer[1] > max_fwer){max_fwer <- fwer[1]; ncp_max_fwer <- ncp;}
  }
for(delta2 in seq(-w2,w2,by=grid_width_2))
  {
    ncp <- c(delta2,0)
    fwer <- get_familywise_Type_I_error_rate(ncp)
    if(fwer[1] > max_fwer){max_fwer <- fwer[1]; ncp_max_fwer <- ncp;}
  }
for(delta in seq(-w1,w1,by=grid_width_1))
  {
    ncp <- c(delta,-(rho1/rho2)*delta)
    fwer <- get_familywise_Type_I_error_rate(ncp)
    if(fwer[1] > max_fwer){max_fwer <- fwer[1]; ncp_max_fwer <- ncp;}
  }

print("Max Familywise Type I Error, corresponding noncentrality parameters, and grid density")
print(c(max_fwer,ncp_max_fwer,grid_width_1,grid_width_2))
