## Generate Discretized LP
library(mvtnorm)

## Input values for discretized optimization problem

max_error_prob <- 0
alpha <- (0.05 - 10^(-4)) ## desired FWER
beta <- (1-0.88) ## 1 minus beta is desired power to reject H0C at ncp_alt defined below
p1<- 0.5 # proportion of population in subpopulation 1
p2<- 1-p1 # proportion of population in subpopulation 2
rho1 <- sqrt(p1) # correlation of Z_1 and Z_C
rho2 <- sqrt(p2) # correlation of Z_2 and Z_C
ncp_alt <- (qnorm(1-alpha) + qnorm(0.9))*c(rho1,rho2) # alternative used in power constraint which is set so that UMP test of H0C alone has power 0.9 to reject, at level alpha. 
delta_1_min <- ncp_alt[1]-(10^(-10)) # set just below alternative used in power constraint, so as to be compatible with loss function and prior below
delta_2_min <- ncp_alt[2]-(10^(-10))
#set widths of large square outside of which multiple testing procedure rejects no null hypotheses (corresponds to w in text)
w1 <- 5  
w2 <- 5
# dimension of small squares used in discretization
tau <- 0.02
number_actions <- 6 # number of actions excluding default action 0
					# These correspond to rejecting the subsets of null hypotheses: 
					# S = (H01,H02,H0C,{H01,H0C},{H02,H0C},{H01,H02,H0C}) 

map_from_P_to_type_I_error_indicator_over_set_of_actions <- function(ncp)
{
	return(c(ncp[1]<=0,ncp[2]<=0,rho1*ncp[1]+rho2*ncp[2]<=0,ncp[1]<=0 || rho1*ncp[1]+rho2*ncp[2]<=0,ncp[2]<=0 || rho1*ncp[1]+rho2*ncp[2]<=0,ncp[1]<=0 || ncp[2]<=0 || rho1*ncp[1]+rho2*ncp[2]<=0))
}

indicator_contribute_to_combined_pop_power <- c(0,0,1,1,1,1)

# Loss function
L <- function(ncp){ # returns vector corresponding to loss function values at each action, for data generating distribution defined by non-centrality parameters ncp
	L1 <- as.numeric(ncp[1]>delta_1_min)*c(1,0,1,1,0,1,0) # indicator of delta_1 > delta_1_min and subset S_j (defined above) containing H_{01}
	L2 <- as.numeric(ncp[2]>delta_2_min)*c(1,1,0,1,1,0,0) # indicator of delta_2 > delta_2_min and subset S_j containing H_{02}
	L_total <- L1+L2
	return(L_total[2:7] - L_total[1]) # subtract L_total[1] since first element represents default option, which we do not allocate a variable for in our LP representation.
}

# For Symmetric Case:
prior_weights <- c(1/4,1/4,1/4,1/4)
prior_support <- c(list(c(0,0)),list(c(ncp_alt[1],0)),list(c(0,ncp_alt[2])),list(ncp_alt))

# list of pairs of non-centrality parameters in G_{tau,w}
ncp_list <- list()
for(x1 in seq(-w1,w2,by=tau)) {ncp_list <- c(ncp_list,list(c(x1,0)))}
for(y in seq(-w1,w2,by=tau)) {ncp_list <- c(ncp_list,list(c(0,y)))}
for(z in seq(-w1/rho2,w2/rho1,by=tau)) {ncp_list <- c(ncp_list,list(c(rho2*z,-rho1*z)))}
ncp_list <- unique(ncp_list)

# Settings for integration region and precision in computing lower bound to original problem using dual solution to discretized problem
max_eval_iters <- 100000
w1_unconst <- 10
w2_unconst <- 10

# Set grid density for grid search used in computing worst-case familywise Type I error rate
# Each rectangle in grid set to have dimensions:
grid_width_1 <- 10^(-4)
grid_width_2 <- 10^(-4)

# construct list of rectangles in set R
list_of_rectangles <- list()
for(x in seq(-w1,w1,by=tau))
{
	for(y in seq(-w2,w2,by=tau))
	{
		list_of_rectangles <- c(list_of_rectangles,list(c(x,y)))	
	}
}
save(list_of_rectangles,file="list_of_rectangles.rdata")
# number of variables in linear program
number_of_variables <- length(list_of_rectangles)*number_actions

## Construct Familywise Type I error constraints
constraint_list <- c()
counter <- 1
max_error_prob <- 0
for(ncp in ncp_list)
{
	rvec <- c()
	## Construct constraint that P_{ncp}(Type I error) \leq \alpha
	true_nulls_violated_by_action_set <- map_from_P_to_type_I_error_indicator_over_set_of_actions(ncp)
	for(r in list_of_rectangles){
		## Get P_{ncp_vec}(Z \in rectangle r)
		prob_r <- pmvnorm(mean=ncp,sigma=diag(2),lower=c(r[1],r[2]),upper=c(r[1]+tau,r[2]+tau))
        if(attr(prob_r,"error") > max_error_prob){max_error_prob <- attr(prob_r,"error")}
		rvec <- c(rvec,prob_r*true_nulls_violated_by_action_set)
	}
	constraint_list <- rbind(constraint_list,rvec)
	if(floor(counter/10) == counter/10) {print(paste(counter,"out of ",length(ncp_list),"familywise Type I error constraints generated"))}
	counter <- counter + 1
}

## Construct Power Constraint
rvec <- c()
# Indicators of rejecting H_0C, based don ordering: (H01,H02,H0C,{H01,H0C},{H02,H0C},{H01,H02,H0C}) 
for(r in list_of_rectangles){
	## Get P_{ncp_vec}(Z in rectangle r)
	prob_r <- pmvnorm(mean=ncp_alt,sigma=diag(2),lower=c(r[1],r[2]),upper=c(r[1]+tau,r[2]+tau))
        if(attr(prob_r,"error") > max_error_prob){max_error_prob <- attr(prob_r,"error")}
	## Get subsets at which H_{0*} rejected
	rvec <- c(rvec,prob_r*indicator_contribute_to_combined_pop_power)
}
constraint_list <- rbind(constraint_list,-rvec)

# construct objective function vector
objective_function_vector <- rep(0,number_of_variables)
counter <- 1
for(ncp in prior_support)
{
	contribution_to_risk <- c()
	loss_function_value <- L(ncp) # returns vector of loss function value for each possible action
	for(r in list_of_rectangles){
		## Get P_{ncp_vec}(Z \in r)
		prob_r <- pmvnorm(mean=ncp,sigma=diag(2),lower=c(r[1],r[2]),upper=c(r[1]+tau,r[2]+tau))
		## Get subsets at which H_{01} rejected	
		if(attr(prob_r,"error") > max_error_prob){max_error_prob <- attr(prob_r,"error")}
	  	contribution_to_risk <- c(contribution_to_risk,prob_r)
	}
	objective_function_vector <- objective_function_vector+as.vector(prior_weights[counter]*loss_function_value%*% t(contribution_to_risk))
	counter <- counter + 1
}

# print estimated maximum error in computing bivariate normal probabilities using pmvnorm:
print("Max Biv. Normal Error")
print(max_error_prob)

# record components of discretized linear program
write(constraint_list,f=paste("A.r"))
write(objective_function_vector,f=paste("c.r"))
write(dim(constraint_list)[1],f=paste("number_dense_constraints.txt"))
write(dim(constraint_list)[2],f=paste("number_variables.txt"))
write(number_actions,f=paste("number_elements_action_set.txt"))
write(alpha,f=paste("alpha.txt"))
write(1-beta,f=paste("one_minus_beta.txt"))

