/*****************************************************************************

  Implementation of the 
  Primal Dual (i.e. Self Dual) Simplex Method
  R. Vanderbei, 3 October 1994

  Solves problems in the form:

  T
  max c x

  A x  = b
  x >= 0

  A is an m by N matrix (it is convenient to reserve n for 
  the difference N-m).  Artificial variables have been 
  added (hence N > m).  One may assume that the last 
  m columns of A are invertible and hence can be used as 
  a starting basis.

 ******************************************************************************/         
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <time.h>

#ifdef QuadPrec
#include "Quad.h"
#define double Quad
#else
#define high(x) (x)
#endif

#include "linalg.h"
#include "lu.h"
#include "myalloc.h"
#include "macros.h"

#define EPS  1.0e-100
#define EPS0 1.0e-100
#define EPS1 1.0e-100
#define EPS2 1.0e-100
#define EPS3 0.0
#define MAX_ITER 10000000


double sdotprod(double *c, double *x_B, int *basics, int m);

void Nt_times_y( 
		int N, 
		double *at, 
		int *iat, 
		int *kat, 
		int *basicflag,
		double *y, 
		int *iy, 
		int ny, 
		double *yN,
		int *iyN,
		int *pnyN
	       );

int ratio_test(
		double *dy, 
		int   *idy,
		int    ndy,
		double *y, 
		double *ybar, 
		double mu
	      );

int solver2(
		int m,		/* number of constraints */
		int n,		/* number of variables */
		int nz,		/* number of nonzeros in sparse constraint matrix */
		int *ia, 		/* array row indices */
		int *ka, 		/* array of indices into ia and a */
		double *a,		/* array of nonzeros in the constraint matrix */
		double *b, 		/* right-hand side */
		double *c,          /* objective coefficients */
		double *c2,         /* objective coefficients */
		double  f, 		/* objective function shift */
		int *basics,
		int *nonbasics,
		int *basicflag,
		int n0,
		int m0
	   );


char* output_name = NULL;

main(int argc, char **argv)
{
	int m;		/* number of constraints */
	int n;		/* number of variables */
	int nz;		/* number of nonzeros in sparse constraint matrix */
	int *ia; 		/* array row indices */
	int *ka; 		/* array of indices into ia and a */
	double *a;		/* array of nonzeros in the constraint matrix */
	double *b; 		/* right-hand side */
	double *c;          /* objective coefficients */
	double *c2;         /* objective coefficients */
	double  f; 		/* objective function shift */
	int     *basics;	/* list of basic variable indices */
	int     *nonbasics;	/* list of non-basic variable indices */
	int     *basicflag; /* neg if nonbasic, pos if basic */
	

	//Prase Inputs into Standard form
	if( argc <  5 ){
		fprintf(stderr,"Usage: ./el1 [A] [b] [c] [#sparse_coeff] (output_name)\n");
		exit(0);
	}

	FILE* fp_A = fopen(argv[1],"r");
	FILE* fp_b = fopen(argv[2],"r");
	FILE* fp_c = fopen(argv[3],"r");
	int num_sparse_coeff = atoi(argv[4]);
	if( argc >= 6 ){
		output_name = argv[5];
	}else{
		output_name = "";
	}
	
	//read b
	double* b0;
	int m0;
	fscanf(fp_b,"%d",&m0);
	MALLOC(b0,m0,double);
	int i=0;
	int j;
	for(i=0;i<m0;i++){
		fscanf(fp_b,"%lf",&(b0[i]));
		//fprintf(stderr,"%f\n",b0[i]);
	}
	fclose(fp_b);
	
	//read c
	double* c0;
	int n0;
	fscanf(fp_c,"%d",&n0);
	MALLOC(c0,n0,double);
	for(i=0;i<n0;i++){
		fscanf(fp_c,"%lf",&(c0[i]));
	}
	fclose(fp_c);
	
	//read A
	double** A;
	MALLOC(A,m0,double*);
	for(i=0;i<m0;i++){
		MALLOC(A[i],n0,double);
		for(j=0;j<n0;j++){
			fscanf(fp_A,"%lf",&(A[i][j]));
		}
	}
	fclose(fp_A);
	
	
	// Define n,m,nnz
	m = m0 + n0/num_sparse_coeff;
	n = n0+m ; //the m0-1 constaint has 2 slack variables
	nz = m0*n0 + n0*num_sparse_coeff/num_sparse_coeff + m;
	
	MALLOC(        a, nz,  double );
	MALLOC(       ia, nz,   int );      
	MALLOC(       ka, n+1,  int );      
	MALLOC(        b, m,   double );      
	MALLOC(        c, n,   double );      
	MALLOC(       c2, n,   double );      
	MALLOC(   basics, m,   int );      
	MALLOC(nonbasics, n-m, int );      
	MALLOC(basicflag, n,   int );      
	
	//convert [[A1;A2] I] to a, ia, ka
	int k=0;

	for(j=0;j<n0;j++){
		ka[j] = k;
		for(i=0;i<m0;i++){
			a[k] = A[i][j];
			ia[k] = i;
			k++;
		}
		a[k] = 1.0;
		ia[k] = m0 + j/num_sparse_coeff;
		k++;
	}
	
	i=0;
	for(j=n0;j<n0+m;j++){
		ka[j] = k;
		a[k] = 1.0;
		ia[k] = i;
		k++;
		i++;
	}
	ka[n] = k;
	
	//convert b0 to b
	for(i=0;i<m0;i++){
		b[i] = b0[i];
	}
	for(i=m0;i<m;i++)
		b[i] = 1.0;
	
	//convert c0 to c
	for(j=0;j<n0;j++){
		c[j] = c0[j];
		c2[j] = -1e-1*drand48();
		//c2[j] = -1;
	}
	for(j=n0;j<n0+m/*+1*/;j++){
		c[j] =  0.0;
		c2[j] = 0.0;
	}
	//c2[n0+m0-1] = 0.0;
	//c[n0+m] = -1000;
	
	int bv=0, nv=0;
	//non-basic
	for(j=0;j<n0;j++){
		nonbasics[nv] = j;
		basicflag[j] = -nv-1;
		nv++;
	}
	
	//basic
	for(j=n0;j<n0+m;j++){
		//if(j==n0+m0-1) // this is non-basic
		//	continue;
		basics[bv]=j;
		basicflag[j] = bv;
		bv++;
	}
	
	int status = solver2(m,n,nz,ia,ka,a,b,c,c2,0,basics,nonbasics,basicflag,n0,m0);
	if( status == 0 ){
		fprintf(stderr,"\nstatus: OPTIMAL\n");
	}else if( status == 1 ){
		fprintf(stderr,"\nstatus: UNBOUNDED\n");
	}else if( status ==  2 ){
		fprintf(stderr,"\nstatus: INFEASIBLE\n");
	}

	return 0;
}

int solver2(
		int m,		/* number of constraints */
		int N,		/* number of variables */
		int nz,		/* number of nonzeros in sparse constraint matrix */
		int *ia, 		/* array row indices */
		int *ka, 		/* array of indices into ia and a */
		double *a,		/* array of nonzeros in the constraint matrix */
		double *b, 		/* right-hand side */
		double *c,          /* objective coefficients */
		double *c2,         /* objective coefficients */
		double  f, 		/* objective function shift */
		int *basics,
		int *nonbasics,
		int *basicflag,
		int n0,
		int m0
	   )
{
	FILE *fp;

	double  *x_B;	/* primal basics */
	double  *y_N;	/* dual nonbasics */

	double  *xbar_B;	/* primal basic perturbation */
	double  *ybar_N;	/* dual nonbasic perturbation */

	double  *dy_N;	/*  dual  basics step direction - values (sparse) */
	int    *idy_N;	/*  dual  basics step direction - row indices */
	int     ndy_N;	/* number of nonz in dy_N */

	double  *dx_B;	/* primal basics step direction - values (sparse) */
	int    *idx_B;	/* primal basics step direction - row indices */
	int     ndx_B;	/* number of nonz in dx_B */

	double  *at;	/* sparse data structure for A^T */
	int    *iat;
	int    *kat;

	int     col_in;	/* entering column; index in 'nonbasics' */
	int     col_out;	/* leaving column; index in 'basics' */

	int     iter = 0;	/* number of iterations */
	int     i,j,k,n,v=0;

	double  s, t, sbar, tbar, mu=HUGE_VAL, old_mu, primal_obj;

	double  *vec;
	int    *ivec;
	int     nvec;

	int     new_pt = TRUE;

	int	    status;

	int     from_scratch;

	/*******************************************************************
	 * For convenience, we put...
	 *******************************************************************/

	n = N-m;

	/*******************************************************************
	 * Read in the Data and initialize the common memory sites.
	 *******************************************************************/

	MALLOC(    x_B, m,   double );      
	MALLOC( xbar_B, m,   double );      
	MALLOC(   dx_B, m,   double );      
	MALLOC(    y_N, n,   double );      
	MALLOC( ybar_N, n,   double );      
	MALLOC(   dy_N, n,   double );      
	MALLOC(    vec, N,   double );
	MALLOC(  idx_B, m,    int );      
	MALLOC(  idy_N, n,    int );      
	MALLOC(   ivec, N,    int );
	MALLOC(     at, nz,  double );
	MALLOC(    iat, nz,   int );
	MALLOC(    kat, m+1,  int );

	/**************************************************************** 
	 *  Initialization.              				    *
	 ****************************************************************/

	atnum(m,N,ka,ia,a,kat,iat,at);

	/*
	   printf("A: \n");
	   for (j=0; j<N; j++) {
	   for (k=ka[j]; k<ka[j+1]; k++) {
	   printf("%4d %4d %10.4f \n", ia[k], j, a[k]);
	   }
	   }
	   printf("basics: \n");
	   for (i=0; i<m; i++) {
	   printf("%4d \n", basics[i]);
	   }
	   printf("nonbasics: \n");
	   for (j=0; j<n; j++) {
	   printf("%4d \n", nonbasics[j]);
	   }
	   */

	lufac( m, ka, ia, a, basics, 0 );
	////////////////////////////////////////////// C
	for (j=0; j<n; j++) {
		y_N[j] = 0;
	}
	nvec = 0;
	for (i=0; i<m; i++) {
		if (c[basics[i]] != 0.0) {
			vec[nvec] = c[basics[i]];
			ivec[nvec] = i;
			nvec++;
		}
	}
	btsolve( m, vec, ivec, &nvec );  		
	Nt_times_y( N, at, iat, kat, basicflag, vec, ivec, nvec, 
			dy_N, idy_N, &ndy_N );
	for (k=0; k<ndy_N; k++) {
		y_N[idy_N[k]] = dy_N[k];
	}
	for (j=0; j<n; j++) {
		y_N[j] -= c[nonbasics[j]];
	}
	/////////////////////////////////////////////// C2
	for (j=0; j<n; j++) {
		ybar_N[j] = 0;
	}
	nvec = 0;
	for (i=0; i<m; i++) {
		if (c2[basics[i]] != 0.0) {
			vec[nvec] = c2[basics[i]];
			ivec[nvec] = i;
			nvec++;
		}
	}
	btsolve( m, vec, ivec, &nvec );  		
	Nt_times_y( N, at, iat, kat, basicflag, vec, ivec, nvec, 
			dy_N, idy_N, &ndy_N );
	for (k=0; k<ndy_N; k++) {
		ybar_N[idy_N[k]] = dy_N[k];
	}
	for (j=0; j<n; j++) {
		ybar_N[j] -= c2[nonbasics[j]];
		if (ybar_N[j] < 0) printf("error: ybar_N[%d] = %e \n", j, ybar_N[j]);
	}

	for (i=0; i<m; i++) {
		x_B[i] = 0;
		//xbar_B[i] = 0.1*drand48();
		xbar_B[i] = 0.0;
	}
	//xbar_B[m0-1] = 0.1*drand48();
	
	nvec = 0;
	for (i=0; i<m; i++) {
		if ( b[i] != 0.0 ) {
			vec[nvec] = b[i];
			ivec[nvec] = i;
			nvec++;
		}
	}
	bsolve( m, vec, ivec, &nvec );
	for (i=0; i<nvec; i++) {
		x_B[ivec[i]] = vec[i];
		//if (vec[i] < 0) printf("error: x_B[%d] = %e \n", i, vec[i]);
	}
	/*
	   printf("x_B: \n");
	   for (i=0; i<m; i++) {
	   printf(" %10.6f \n", x_B[i]);
	   }
	   printf("y_N, ybar_N: \n");
	   for (j=0; j<n; j++) {
	   printf(" %10.6f %10.6f \n", y_N[j], ybar_N[j]);
	   }
	   */

	printf ("m = %d,n = %d,nz = %d\n",m,N,nz);
	printf(
			"---------------------------------------------------------------------------\n"
			"          |   Primal      |        |                           arithmetic  \n"
			"  Iter    |  Obj Value    |   mu   |   nonz(L)     nonz(U)     operations  \n"
			"- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -\n"
	      );
	fflush(stdout);

	/****************************************************************
	 * 	Main loop                                                   *
	 ****************************************************************/
	
	for (iter=0; iter<MAX_ITER; iter++) {

		/*************************************************************
		 * STEP 1: Find mu                                            *
		 *************************************************************/
		old_mu = mu;
		mu = -HUGE_VAL;
		col_in  = -1;
		for (j=0; j<n; j++) {
			if (ybar_N[j] > EPS2) { 
				if ( mu < -y_N[j]/ybar_N[j] ) {
					mu = -y_N[j]/ybar_N[j];
					col_in  = j;
				}
			}
		}
		col_out = -1;
		for (i=0; i<m; i++) {
			if (xbar_B[i] > EPS2) {
				if ( mu < -x_B[i]/xbar_B[i] ) {
					mu = -x_B[i]/xbar_B[i];
					col_out = i;
					col_in  = -1;
				}
			}
		}

		/*************************************************************
		 * STEP 0: Record current portfolio                           *
		 *************************************************************/

		/*
		   if (mu + EPS3 < old_mu) {
		   printf("mu = %f \n", mu);
		   for (i=0; i<m; i++) {
		   if (basics[i] < n0 && x_B[i] > EPS0) {
		   printf("       %6d\t%10.6f \n", basics[i],x_B[i]);
		   }
		   else if (basics[i] < 2*n0 && x_B[i] > EPS0) {
		   printf("       %6d\t%10.6f \n", basics[i],-x_B[i]);
		   }
		   }
		   }
		   */

		primal_obj = sdotprod(c,x_B,basics,m) + f;
		if ( mu <= EPS3 ) {	/* OPTIMAL */

			printf("mu = %f \n", mu);

			char filename[1000];

			sprintf(filename,"%s.sol.txt",output_name);
			FILE* fp_sol = fopen(filename,"w");
			for (i=0; i<m; i++) {
				if (basics[i] < n0 && x_B[i] > EPS0) {
					
					fprintf(fp_sol,"%d\t%10.6f \n", basics[i]+1,x_B[i]);
				}
			}
			fclose(fp_sol);
			
			sprintf(filename,"%s.dual.dense.txt",output_name);
			FILE* fp_dual_dense = fopen(filename,"w");
			sprintf(filename,"%s.dual.sparse.txt",output_name);
			FILE* fp_dual_sparse = fopen(filename,"w");
			for (i=0; i<n; i++) {
				if( nonbasics[i]>=n0 && nonbasics[i] < n0+m0 )
					fprintf(fp_dual_dense, "%d\t%g \n", nonbasics[i]-n0+1,y_N[i]);
				else if( nonbasics[i]>=n0 && nonbasics[i] >= n0+m0 )
					fprintf(fp_dual_sparse, "%d\t%g \n", nonbasics[i]-n0-m0+1,y_N[i]);
			}
			fclose(fp_dual_dense);
			fclose(fp_dual_sparse);		
			
			status = 0;
			break;
		}
		
		if ( col_out >= 0 ) {

			/*************************************************************
			 *                          -1  T                             *
			 * STEP 2: Compute dy  = -(B  N) e                            * 
			 *                   N            i			     *
			 *         where i = col_out                                  *
			 *************************************************************/

			vec[0] = -1.0;
			ivec[0] = col_out;
			nvec = 1;

			btsolve( m, vec, ivec, &nvec );  		

			Nt_times_y( N, at, iat, kat, basicflag, vec, ivec, nvec, 
					dy_N, idy_N, &ndy_N );

			/*************************************************************
			 * STEP 3: Ratio test to find entering column                 * 
			 *************************************************************/

			col_in = ratio_test( dy_N, idy_N, ndy_N, y_N, ybar_N, mu );

			if (col_in == -1) { 	/* INFEASIBLE */
				status = 2;
				break;
			}

			/*************************************************************
			 *                        -1                                  *
			 * STEP 4: Compute dx  = B  N e                               * 
			 *                   B         j                              *
			 *                                                            *
			 *************************************************************/

			j = nonbasics[col_in];
			for (i=0, k=ka[j]; k<ka[j+1]; i++, k++) {
				dx_B[i] =  a[k];
				idx_B[i] = ia[k];
			}
			ndx_B = i;

			bsolve( m, dx_B, idx_B, &ndx_B );

		} else {

			/*************************************************************
			 *                        -1                                  *
			 * STEP 2: Compute dx  = B  N e                               * 
			 *                   B         j                              *
			 *************************************************************/

			j = nonbasics[col_in];
			for (i=0, k=ka[j]; k<ka[j+1]; i++, k++) {
				dx_B[i] =  a[k];
				idx_B[i] = ia[k];
			}
			ndx_B = i;

			bsolve( m, dx_B, idx_B, &ndx_B );

			/*************************************************************
			 * STEP 3: Ratio test to find leaving column                  * 
			 *************************************************************/

			col_out = ratio_test( dx_B, idx_B, ndx_B, x_B, xbar_B, mu );

			if (col_out == -1) {	/* UNBOUNDED */
				status = 1;
				break;
			}

			/*************************************************************
			 *                          -1  T                             *
			 * STEP 4: Compute dy  = -(B  N) e                            * 
			 *                   N            i			     *
			 *                                                            *
			 *************************************************************/

			vec[0] = -1.0;
			ivec[0] = col_out;
			nvec = 1;

			btsolve( m, vec, ivec, &nvec );  		

			Nt_times_y( N, at, iat, kat, basicflag, vec, ivec, nvec, 
					dy_N, idy_N, &ndy_N );

		}

		/*************************************************************
		 *                                                            *
		 * STEP 5: Put       t = x /dx                                *
		 *                        i   i                               *
		 *                   _   _                                    *
		 *                   t = x /dx                                *
		 *                        i   i                               *
		 *                   s = y /dy                                *
		 *                        j   j                               *
		 *                   _   _                                    *
		 *                   s = y /dy                                *
		 *                        j   j                               *
		 *************************************************************/

		/* this is inefficient - it should be fixed */
		for (k=0; k<ndx_B; k++) if (idx_B[k] == col_out) break;

		t    =    x_B[col_out]/dx_B[k];
		tbar = xbar_B[col_out]/dx_B[k];

		/* this is inefficient - it should be fixed */
		for (k=0; k<ndy_N; k++) if (idy_N[k] == col_in) break;

		s    =    y_N[col_in]/dy_N[k];
		sbar = ybar_N[col_in]/dy_N[k];

		/*************************************************************
		 *                                _    _    _                 *
		 * STEP 7: Set y  = y  - s dy     y  = y  - s dy              *
		 *              N    N       N     N    N       N             *
		 *                                _    _                      *
		 *             y  = s             y  = s                      *
		 *              i                  i                          *
		 *             _    _    _                                    *
		 *             x  = x  - t dx     x  = x  - t dx              *
		 *              B    B       B     B    B       B             *
		 *             _    _                                         *
		 *             x  = t             x  = t                      *
		 *              j                  j                          *
		 *************************************************************/

		// printf("entering = %d, leaving = %d \n", col_in, col_out);
		for (k=0; k<ndy_N; k++) {
			j = idy_N[k];
			y_N[j]    -= s   *dy_N[k];
			ybar_N[j] -= sbar*dy_N[k];

			// printf("dy_N[%3d] = %12.5e \n", j, dy_N[k]);
		}
		/*
		   for (j=0; j<n; j++) {
		   printf("y_N[%3d] = %12.5e, ybar_N[%3d] = %12.5e \n", 
		   j, y_N[j], j, ybar_N[j]);
		   }
		   */

		y_N[col_in]    = s;
		ybar_N[col_in] = sbar;

		for (k=0; k<ndx_B; k++) {
			i = idx_B[k];
			x_B[i]    -= t   *dx_B[k];
			xbar_B[i] -= tbar*dx_B[k];

			// printf("dx_B[%3d] = %12.5e \n", i, dx_B[k]);
		}
		/*
		   for (i=0; i<m; i++) {
		   printf("x_B[%3d] = %12.5e, xbar_B[%3d] = %12.5e \n", 
		   i, x_B[i], i, xbar_B[i]);
		   }
		   */

		x_B[col_out]     = t;
		xbar_B[col_out]  = tbar;

		/*************************************************************
		 * STEP 8: Update basis                                       * 
		 *************************************************************/

		i =    basics[col_out];
		j = nonbasics[col_in];
		basics[col_out]   = j;
		nonbasics[col_in] = i;
		basicflag[i] = -col_in-1;
		basicflag[j] = col_out;

		/*************************************************************
		 * STEP 9: Refactor basis and print statistics                *
		 *************************************************************/
		
		from_scratch = refactor( m, ka, ia, a, basics, col_out, v );

		if (from_scratch) {
			primal_obj = sdotprod(c,x_B,basics,m) + f;
			printf("%8d   %14.7e %9.2e \n", iter, high(primal_obj), high(mu) );
			fflush(stdout);
		}
	
	} 

	primal_obj = sdotprod(c,x_B,basics,m) + f;
	printf("\n");
	printf("%8d   %14.14e %9.2e \n", iter, high(primal_obj), high(mu) );

	/****************************************************************
	 * 	Transcribe solution to x vector and dual solution to y      *
	 ****************************************************************/

	/*
	   for (j=0; j<N; j++) x[j] = 0.0;
	   for (i=0; i<m; i++) x[basics[i]] = x_B[i];

	   for (j=0; j<N; j++) y[j] = 0.0;
	   for (i=0; i<n; i++) y[nonbasics[i]] = y_N[i];
	   */

	/****************************************************************
	 * 	Split out slack variables and shift dual variables.
	 ****************************************************************/

	/*
	   for (j=0; j<n; j++) z[j] = y[j];
	   for (i=0; i<m; i++) {
	   y[i] = y[n+i];
	   w[i] = x[n+i];
	   }
	   */

	/****************************************************************
	 * 	Free work space                                             *
	 ****************************************************************/

	FREE(  vec );
	FREE( ivec );
	FREE(  x_B );
	FREE(  y_N );
	FREE( dx_B );
	FREE(idx_B );
	FREE( dy_N );
	FREE(idy_N );
	FREE( nonbasics );
	FREE( basics );

	return status;

}   /* End of solver */

void Display_Solution(int m,int *basics,double *X)
{
	int i;

	printf("SOLUTION:\n\n");
	for (i=0;i<m;i++)
		printf("  X[%d] = %lf\n",basics[i], high(X[i]) );
}

void Nt_times_y( 
		int n, 
		double *at, 
		int *iat, 
		int *kat, 
		int *basicflag,
		double *y, 
		int *iy, 
		int ny, 
		double *yN,
		int *iyN,
		int *pnyN
	       )
{
	int i,j,jj,k,kk;

	static double *a=NULL;
	static int  *tag=NULL;
	static int *link=NULL;
	static int  currtag=1;

	if (  a  == NULL) MALLOC(  a,  n,   double);
	if ( tag == NULL) CALLOC( tag, n,   int);
	if (link == NULL) {CALLOC(link, n+2, int); link++;}

	jj = -1;
	for (k=0; k<ny; k++) {
		i = iy[k];
		for (kk=kat[i]; kk<kat[i+1]; kk++) {
			j = iat[kk];
			if (basicflag[j] < 0) {
				if (tag[j] != currtag) {
					a[j] = 0.0;
					tag[j] = currtag;
					link[jj] = j;
					jj = j;
				}
				a[j] += y[k]*at[kk];
			}
		}
	}
	link[jj] = n;
	currtag++;

	k = 0;
	for (jj=link[-1]; jj<n; jj=link[jj]) {
		if ( ABS(a[jj]) > EPS1 ) {
			yN[k] = a[jj];
			iyN[k] = -basicflag[jj]-1;
			k++;
		}
	}
	*pnyN = k;
}

int ratio_test(
		double *dy, 
		int   *idy,
		int    ndy,
		double *y, 
		double *ybar, 
		double mu
	      )
{
	int j, jj = -1, k, kk;
	double min = HUGE_VAL;

	for (k=0; k<ndy; k++) {
		if ( dy[k] > EPS1 ) {
			j = idy[k];
			if ( (y[j] + mu*ybar[j])/dy[k] < min ) {
				min = (y[j] + mu*ybar[j])/dy[k];
				jj = j;
				kk = k;
			}
		}
	}

	return jj;
}

double sdotprod(double *c, double *x_B, int *basics, int m)
{
	int i;
	double prod = 0.0;

	for (i=0; i<m; i++) { prod += c[basics[i]]*x_B[i]; }

	return prod;
}
